# ******************************************************
#
#   Script with examples to run the foram3D R package 
#   
#   Brombacher, A., Searle-Barnes, A., Zhang, W., Ezard, T.H.G.
#   Analysing planktonic foraminiferal growth in three dimensions: 
#   an R package for automated trait measurements from CT scans.
#   
#   Two example data sets are included: 'Mlimbata' containing
#   centroid data from a Menardella limbata specimen, 
#   sample ID 154-925C-10H-4W 115-117 cm, and 'Tsacculifer' containing 
#   centroid data from a Trilobatus sacculifer specimen, collected 
#   during the Palaeogene GLObal Warming events (GLOW) cruise by
#   box core from GLOW Station 8 (9 21’ 25.20” S; 40 35’ 27.60” E).
#   
#   Submitted to the Journal of Micropalaeontology
#
#   August 2022
#
# ******************************************************

# Installing the foram3D package from Github -----------------------------------------------------

## The 'devtools' package enables package download from github
install.packages("devtools")     
library(devtools)

## The package requires R version 4.2.0 or higher. 
## Download the latest version of R here if necessary: https://cran.r-project.org/

## Install foram3D package from Anieke Brombacher's github account.
install_github('AniekeBrombacher/foram3D')
library(foram3D)



# Interactive 3D plots of the centroid data ------------------------------------------------------
# Examples provided use the 'Mlimbata' and 'Tsacculifer' data sets included in the package
## 'rgl' is a package to generate interactive 3D plots
## Note for MacOS users: install XQuartz first. Download here: https://www.xquartz.org/
install.packages('rgl')
library(rgl)

## Plot centroids in 3D space
# M. limbata
open3d()     # Open interactive 3D window
points3d(Mlimbata$CentroidX, Mlimbata$CentroidY, Mlimbata$CentroidZ)
lines3d(Mlimbata$CentroidX, Mlimbata$CentroidY, Mlimbata$CentroidZ)
text3d(Mlimbata$CentroidX, Mlimbata$CentroidY, Mlimbata$CentroidZ, Mlimbata$Chamber)
## click and hold to rotate, scroll to zoom

# T. sacculifer
open3d()     # Open interactive 3D window
points3d(Tsacculifer$CentroidX, Tsacculifer$CentroidY, Tsacculifer$CentroidZ)
lines3d(Tsacculifer$CentroidX, Tsacculifer$CentroidY, Tsacculifer$CentroidZ)
text3d(Tsacculifer$CentroidX, Tsacculifer$CentroidY, Tsacculifer$CentroidZ, Tsacculifer$Chamber)
## click and hold to rotate, scroll to zoom



# Chamber ordering and checking ------------------------------------------------------------------

# Examples provided use the 'Mlimbata' dataset included in the package
# Swap 'Mlimbata' with 'Tsacculifer' to see the results for T. sacculifer

# Check the chamber order of the original data set
with(Mlimbata, check.chamber.order(X=CentroidX, Y=CentroidY, Z=CentroidZ))

# Create 'wrong' chamber order on purpose for algorithm to fix
# Here we order the specimen by increasing chamber volume
Mlimbatawrong <- Mlimbata[order(Mlimbata$Volume),]    # order chambers by volume 

# Check the chamber order of chambers ordered by volume
with(Mlimbatawrong, check.chamber.order(X=CentroidX, Y=CentroidY, Z=CentroidZ))

# view data set of the new 'wrong' chamber order
open3d()                                              
with(Mlimbatawrong, points3d(x=CentroidX, y=CentroidY, z=CentroidZ))       
with(Mlimbatawrong, lines3d(x=CentroidX, y=CentroidY, z=CentroidZ))  
with(Mlimbatawrong, text3d(x=CentroidX, y=CentroidY, z=CentroidZ, texts=Chamber))  

# Put chambers in the correct order with package algorithm
Mlimbataordered <- with(Mlimbatawrong, order.chambers(X=CentroidX, Y=CentroidY, Z=CentroidZ, V=Volume))

# View corrected chamber order in interactive plot
open3d()                                               
with(Mlimbataordered, points3d(x=`X centroid`, y=`Y centroid`, z=`Z centroid`))  
with(Mlimbataordered, lines3d(x=`X centroid`, y=`Y centroid`, z=`Z centroid`)) 
with(Mlimbataordered, text3d(x=`X centroid`, y=`Y centroid`, z=`Z centroid`, texts=`Chamber number`)) 



# Trait functions---------------------------------------------------------------------------------

# Examples provided use the 'Mlimbata' and 'Tsacculifer' data sets. 

## Angles between chambers
# Generate angles between subsequent chambers
anglesM <- with(Mlimbata, chamber.angles(X=CentroidX, Y=CentroidY, Z=CentroidZ))    
anglesT <- with(Tsacculifer, chamber.angles(X=CentroidX, Y=CentroidY, Z=CentroidZ))   
# Plot results through ontogeny
par(mfrow=c(1,2))
plot(Mlimbata$Chamber, anglesM) 
plot(Tsacculifer$Chamber, anglesT)

## Trochospirality
# Generate trochospirality angles
trochoM <- with(Mlimbata, trochospirality(X=CentroidX, Y=CentroidY, Z=CentroidZ))   
trochoT <- with(Tsacculifer, trochospirality(X=CentroidX, Y=CentroidY, Z=CentroidZ))    
# Plot results through ontogeny
par(mfrow=c(1,2))
plot(Mlimbata$Chamber, trochoM)     
plot(Tsacculifer$Chamber, trochoT)

## Chambers in final whorl
# Generate chambers in final whorl at the time each chamber was built.
cfwM <- with(Mlimbata, chambers.in.whorl(X=CentroidX, Y=CentroidY, Z=CentroidZ))       
cfwT <- with(Tsacculifer, chambers.in.whorl(X=CentroidX, Y=CentroidY, Z=CentroidZ))  
# Plot chambers in final whorl through ontogeny
par(mfrow=c(1,2))
plot(Mlimbata$Chamber, cfwM[,2])     
plot(Tsacculifer$Chamber, cfwT[,2])

## Number of whorls
# Determine which whorl each chamber belongs to
whorlsM <- with(Mlimbata, whorl(X=CentroidX, Y=CentroidY, Z=CentroidZ)) 
whorlsT <- with(Tsacculifer, whorl(X=CentroidX, Y=CentroidY, Z=CentroidZ)) 
# View results in separate table
View(whorlsM)          
View(whorlsT)

## Coiling direction
# Determine coiling direction of each chamber
cdM <- with(Mlimbata, coiling.direction(X=CentroidX, Y=CentroidY, Z=CentroidZ))     
cdT <- with(Tsacculifer, coiling.direction(X=CentroidX, Y=CentroidY, Z=CentroidZ)) 
# View results 
cdM
cdT

## Data frame with all function results
# Determine angles, trochospirality, chambers in the final whorl, number of whorls 
# and coiling direction in 1 function.
Mlim <- with(Mlimbata, foram.growth.3D(n=Chamber, X=CentroidX, Y=CentroidY, Z=CentroidZ,  proloculus=TRUE))   
Tsac <- with(Tsacculifer, foram.growth.3D(n=Chamber, X=CentroidX, Y=CentroidY, Z=CentroidZ,  proloculus=TRUE))
# View results in separate table
View(Mlim) 
View(Tsac)
# Save data as csv file
write.csv(Mlim, file='Mlimbata3Dgrowth.csv')         
write.csv(Tsac, file='Tsacculifer3Dgrowth.csv')    
